==> dlr-engine-0.9.0-sources.jar: ai/djl/dlr/engine/DlrEngine.java <==
/*
 * Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance
 * with the License. A copy of the License is located at
 *
 * http://aws.amazon.com/apache2.0/
 *
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
 * OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */
package ai.djl.dlr.engine;

import ai.djl.Device;
import ai.djl.Model;
import ai.djl.dlr.jni.JniUtils;
import ai.djl.dlr.jni.LibUtils;
import ai.djl.engine.Engine;
import ai.djl.engine.EngineException;
import ai.djl.ndarray.NDManager;
import ai.djl.training.GradientCollector;

/**
 * The {@code DlrEngine} is an implementation of the {@link Engine} based on the <a
 * href="https://github.com/neo-ai/neo-ai-dlr">Neo DLR</a>.
 *
 * <p>To get an instance of the {@code DlrEngine} when it is not the default Engine, call {@link
 * Engine#getEngine(String)} with the Engine name "DLR".
 */
public final class DlrEngine extends Engine {

    public static final String ENGINE_NAME = "DLR";

    private Engine alternativeEngine;

    private DlrEngine() {}

    static Engine newInstance() {
        try {
            LibUtils.loadLibrary();
            return new DlrEngine();
        } catch (Throwable t) {
            throw new EngineException("Failed to load DLR native library", t);
        }
    }

    private Engine getAlternativeEngine() {
        if (alternativeEngine == null) {
            Engine engine = Engine.getInstance();
            if (engine.getRank() < getRank()) {
                // alternativeEngine should not have the same rank as DLR
                alternativeEngine = engine;
            }
        }
        return alternativeEngine;
    }

    /** {@inheritDoc} */
    @Override
    public String getEngineName() {
        return ENGINE_NAME;
    }

    /** {@inheritDoc} */
    @Override
    public int getRank() {
        return 10;
    }

    /** {@inheritDoc} */
    @Override
    public String getVersion() {
        return JniUtils.getDlrVersion();
    }

    /** {@inheritDoc} */
    @Override
    public boolean hasCapability(String capability) {
        return false;
    }

    /** {@inheritDoc} */
    @Override
    public Model newModel(String name, Device device) {
        return new DlrModel(name, newBaseManager(device), Device.defaultIfNull(device, this));
    }

    /** {@inheritDoc} */
    @Override
    public NDManager newBaseManager() {
        return newBaseManager(null);
    }

    /** {@inheritDoc} */
    @Override
    public NDManager newBaseManager(Device device) {
        if (getAlternativeEngine() != null) {
            return alternativeEngine.newBaseManager(device);
        }
        return DlrNDManager.getSystemManager().newSubManager(device);
    }

    /** {@inheritDoc} */
    @Override
    public GradientCollector newGradientCollector() {
        throw new UnsupportedOperationException("Not supported for DLR");
    }

    /** {@inheritDoc} */
    @Override
    public void setRandomSeed(int seed) {
        throw new UnsupportedOperationException("Not supported for DLR");
    }

    /** {@inheritDoc} */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(200);
        sb.append(getEngineName()).append(':').append(getVersion()).append(", ");
        if (alternativeEngine != null) {
            sb.append("Alternative engine: ").append(alternativeEngine.getEngineName());
        } else {
            sb.append("No alternative engine found");
        }
        return sb.toString();
    }
}

==> dlr-engine-0.11.0-sources.jar: ai/djl/dlr/engine/DlrEngine.java <==
/*
 * Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance
 * with the License. A copy of the License is located at
 *
 * http://aws.amazon.com/apache2.0/
 *
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
 * OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */
package ai.djl.dlr.engine;

import ai.djl.Device;
import ai.djl.Model;
import ai.djl.dlr.jni.JniUtils;
import ai.djl.dlr.jni.LibUtils;
import ai.djl.engine.Engine;
import ai.djl.engine.EngineException;
import ai.djl.ndarray.NDManager;
import ai.djl.nn.SymbolBlock;
import ai.djl.training.GradientCollector;

/**
 * The {@code DlrEngine} is an implementation of the {@link Engine} based on the <a
 * href="https://github.com/neo-ai/neo-ai-dlr">Neo DLR</a>.
 *
 * <p>To get an instance of the {@code DlrEngine} when it is not the default Engine, call {@link
 * Engine#getEngine(String)} with the Engine name "DLR".
 */
public final class DlrEngine extends Engine {

    public static final String ENGINE_NAME = "DLR";
    static final int RANK = 10;

    private Engine alternativeEngine;

    private DlrEngine() {}

    static Engine newInstance() {
        try {
            LibUtils.loadLibrary();
            return new DlrEngine();
        } catch (Throwable t) {
            throw new EngineException("Failed to load DLR native library", t);
        }
    }

    private Engine getAlternativeEngine() {
        if (Boolean.getBoolean("ai.djl.dlr.disable_alternative")) {
            return null;
        }
        if (alternativeEngine == null) {
            Engine engine = Engine.getInstance();
            if (engine.getRank() < getRank()) {
                // alternativeEngine should not have the same rank as DLR
                alternativeEngine = engine;
            }
        }
        return alternativeEngine;
    }

    /** {@inheritDoc} */
    @Override
    public String getEngineName() {
        return ENGINE_NAME;
    }

    /** {@inheritDoc} */
    @Override
    public int getRank() {
        return RANK;
    }

    /** {@inheritDoc} */
    @Override
    public String getVersion() {
        return JniUtils.getDlrVersion();
    }

    /** {@inheritDoc} */
    @Override
    public boolean hasCapability(String capability) {
        return false;
    }

    /** {@inheritDoc} */
    @Override
    public SymbolBlock newSymbolBlock(NDManager manager) {
        throw new UnsupportedOperationException("DLR does not support empty SymbolBlock");
    }

    /** {@inheritDoc} */
    @Override
    public Model newModel(String name, Device device) {
        return new DlrModel(name, newBaseManager(device), Device.defaultIfNull(device, this));
    }

    /** {@inheritDoc} */
    @Override
    public NDManager newBaseManager() {
        return newBaseManager(null);
    }

    /** {@inheritDoc} */
    @Override
    public NDManager newBaseManager(Device device) {
        if (getAlternativeEngine() != null) {
            return alternativeEngine.newBaseManager(device);
        }
        return DlrNDManager.getSystemManager().newSubManager(device);
    }

    /** {@inheritDoc} */
    @Override
    public GradientCollector newGradientCollector() {
        throw new UnsupportedOperationException("Not supported for DLR");
    }

    /** {@inheritDoc} */
    @Override
    public void setRandomSeed(int seed) {
        throw new UnsupportedOperationException("Not supported for DLR");
    }

    /** {@inheritDoc} */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(200);
        sb.append(getEngineName()).append(':').append(getVersion()).append(", ");
        if (alternativeEngine != null) {
            sb.append("Alternative engine: ").append(alternativeEngine.getEngineName());
        } else {
            sb.append("No alternative engine found");
        }
        return sb.toString();
    }
}
